MODULE usrdef
   !!==============================================================================
   !!                       ***  MODULE usrdef   ***
   !! User defined module: used like example to define domain, init, sbc, ...
   !!==============================================================================
   !! History :  NEMO ! 2016-03  (S. Flavoni) 
   !!----------------------------------------------------------------------

   !!----------------------------------------------------------------------
   !!   usr_def_hgr       : initialize the horizontal mesh 
   !!   usr_def_ini       : initial state 
   !!----------------------------------------------------------------------
   USE step_oce       ! module used in the ocean time stepping module (step.F90)
   USE mppini         ! shared/distributed memory setting (mpp_init routine)             
   USE phycst         ! physical constants 
   IMPLICIT NONE
   PRIVATE


   PUBLIC usr_def_hgr
   PUBLIC usr_def_ini

   !!----------------------------------------------------------------------
   !! NEMO/OPA 3.7 , NEMO Consortium (2014)
   !! $Id:$ 
   !! Software governed by the CeCILL licence     (NEMOGCM/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------
CONTAINS

   SUBROUTINE usr_def_hgr( kbench, k_cfg , kff    , pff   , & ! Coriolis parameter  (if domain not on the sphere)
      &                    pglamt, pglamu, pglamv , pglamf, & ! gridpoints position (required)
      &                    pgphit, pgphiu, pgphiv , pgphif, & !         
      &                    pe1t  , pe1u  , pe1v   , pe1f  , & ! scale factors (required)
      &                    pe2t  , pe2u  , pe2v   , pe2f  , & !
      &                    ke1e2u_v                       )   ! u- & v-surfaces (if gridsize reduction is used in strait(s))
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE usr_def_hgr  ***
      !!
      !! ** Purpose :   user defined mesh and Coriolis parameter
      !!
      !! ** Method  :   set all intent(out) argument to a proper value
      !!
      !!                Here GYRE configuration :
      !!          Rectangular mid-latitude domain 
      !!          - with axes rotated by 45 degrees
      !!          - a constant horizontal resolution of 106 km 
      !!          - on a beta-plane
      !!
      !! ** Action  : - define longitude & latitude of t-, u-, v- and f-points (in degrees) 
      !!              - define coriolis parameter at f-point if the domain in not on the sphere (on beta-plane)
      !!              - define i- & j-scale factors at t-, u-, v- and f-points (in meters)
      !!              - define u- & v-surfaces (if gridsize reduction is used in some straits) (in m2)
      !!----------------------------------------------------------------------
      INTEGER                 , INTENT(in   ) ::   kbench, k_cfg   ! parameter of namelist for benchmark, and dimension of GYRE
      INTEGER                 , INTENT(  out) ::   kff             ! =1 Coriolis parameter computed here, =0 otherwise
      REAL(wp), DIMENSION(:,:), INTENT(  out) ::   pff             ! Coriolis factor at f-point                  [1/s]
      REAL(wp), DIMENSION(:,:), INTENT(  out) ::   pglamt, pglamu, pglamv, pglamf !  longitude outputs        [degrees]
      REAL(wp), DIMENSION(:,:), INTENT(  out) ::   pgphit, pgphiu, pgphiv, pgphif !  latitude outputs         [degrees]
      REAL(wp), DIMENSION(:,:), INTENT(  out) ::   pe1t, pe1u, pe1v, pe1f   !  i-scale factors                      [m]
      REAL(wp), DIMENSION(:,:), INTENT(  out) ::   pe2t, pe2u, pe2v, pe2f   !  j-scale factors                      [m]
      INTEGER                 , INTENT(  out) ::   ke1e2u_v                 ! =1 u- & v-surfaces read here, =0 otherwise 
      !!-------------------------------------------------------------------------------
      INTEGER  ::   ji, jj               ! dummy loop indices
      REAL(wp) ::   zlam1, zlam0, zcos_alpha, zim1 , zjm1 , ze1  , ze1deg, zf0 ! local scalars
      REAL(wp) ::   zphi1, zphi0, zsin_alpha, zim05, zjm05, zbeta, znorme      ! local scalars
      !!-------------------------------------------------------------------------------
      !
      IF( nn_timing == 1 )  CALL timing_start('usr_def_hgr')
      !
      !==  beta-plane with regular grid-spacing and rotated domain ==! (GYRE configuration)
      !
      IF(lwp) WRITE(numout,*)
      IF(lwp) WRITE(numout,*) '          beta-plane with regular grid-spacing and rotated domain (GYRE configuration)'
      !
      ! angle 45deg and ze1=106.e+3 / k_cfg forced -> zlam1 = -85deg, zphi1 = 29degN
      zlam1 = -85._wp
      zphi1 =  29._wp
      ! resolution in meters
      ze1 = 106000. / REAL( k_cfg , wp )
      ! benchmark: forced the resolution to be about 100 km
            IF( kbench /= 0 )   ze1 = 106000._wp   
      zsin_alpha = - SQRT( 2._wp ) * 0.5_wp
      zcos_alpha =   SQRT( 2._wp ) * 0.5_wp
      ze1deg = ze1 / (ra * rad)
      IF( kbench /= 0 )   ze1deg = ze1deg / REAL( jp_cfg , wp )   ! benchmark: keep the lat/+lon
      !                                                           ! at the right jp_cfg resolution
      zlam0 = zlam1 + zcos_alpha * ze1deg * REAL( jpjglo-2 , wp )
      zphi0 = zphi1 + zsin_alpha * ze1deg * REAL( jpjglo-2 , wp )
      !   
      IF( nprint==1 .AND. lwp )   THEN
         WRITE(numout,*) '          ze1', ze1, 'cosalpha', zcos_alpha, 'sinalpha', zsin_alpha
         WRITE(numout,*) '          ze1deg', ze1deg, 'zlam0', zlam0, 'zphi0', zphi0
      ENDIF
      !   
      DO jj = 1, jpj 
         DO ji = 1, jpi 
            zim1 = REAL( ji + nimpp - 1 ) - 1.   ;   zim05 = REAL( ji + nimpp - 1 ) - 1.5 
            zjm1 = REAL( jj + njmpp - 1 ) - 1.   ;   zjm05 = REAL( jj + njmpp - 1 ) - 1.5 
            !   
            !glamt(i,j) longitude at T-point
            !gphit(i,j) latitude at T-point  
            pglamt(ji,jj) = zlam0 + zim05 * ze1deg * zcos_alpha + zjm05 * ze1deg * zsin_alpha
            pgphit(ji,jj) = zphi0 - zim05 * ze1deg * zsin_alpha + zjm05 * ze1deg * zcos_alpha
            !   
            !glamu(i,j) longitude at U-point
            !gphiu(i,j) latitude at U-point
            pglamu(ji,jj) = zlam0 + zim1  * ze1deg * zcos_alpha + zjm05 * ze1deg * zsin_alpha
            pgphiu(ji,jj) = zphi0 - zim1  * ze1deg * zsin_alpha + zjm05 * ze1deg * zcos_alpha
            !   
            !glamv(i,j) longitude at V-point
            !gphiv(i,j) latitude at V-point
            pglamv(ji,jj) = zlam0 + zim05 * ze1deg * zcos_alpha + zjm1  * ze1deg * zsin_alpha
            pgphiv(ji,jj) = zphi0 - zim05 * ze1deg * zsin_alpha + zjm1  * ze1deg * zcos_alpha
            !
            !glamf(i,j) longitude at F-point
            !gphif(i,j) latitude at F-point 
            pglamf(ji,jj) = zlam0 + zim1  * ze1deg * zcos_alpha + zjm1  * ze1deg * zsin_alpha
            pgphif(ji,jj) = zphi0 - zim1  * ze1deg * zsin_alpha + zjm1  * ze1deg * zcos_alpha
         END DO
      END DO
      !
      !       !== Horizontal scale factors ==! (in meters)
      !                     
      !                             ! constant grid spacing
      pe1t(:,:) =  ze1     ;      pe2t(:,:) = ze1
      pe1u(:,:) =  ze1     ;      pe2u(:,:) = ze1
      pe1v(:,:) =  ze1     ;      pe2v(:,:) = ze1
      pe1f(:,:) =  ze1     ;      pe2f(:,:) = ze1
      !                             ! NO reduction of grid size in some straits 
      ke1e2u_v = 0                  !    ==>> u_ & v_surfaces will be computed in dom_ghr routine
      !
      !

      !                      !==  Coriolis parameter  ==!
      kff = 1                                                           !  indicate not to compute ff afterward
      !
      zbeta = 2. * omega * COS( rad * zphi1 ) / ra                      ! beta at latitude zphi1
      !SF we overwrite zphi0 (south point in latitude) used just above to define pgphif (value of zphi0=15.5190567531966)
      !SF for computation of coriolis we keep the parameter of Hazeleger, W., and S. S. Drijfhout, JPO 1998.
      zphi0 = 15._wp                                                    !  latitude of the most southern grid point  
      zf0   = 2. * omega * SIN( rad * zphi0 )                           !  compute f0 1st point south
      !
      pff(:,:) = ( zf0 + zbeta * ABS( gphif(:,:) - zphi0 ) * rad * ra ) ! f = f0 +beta* y ( y=0 at south)
      !
      IF(lwp) WRITE(numout,*) '                           beta-plane used. beta = ', zbeta, ' 1/(s.m)'
      !
      IF( nn_timing == 1 )  CALL timing_stop('usr_def_hgr')
      !
   END SUBROUTINE usr_def_hgr


   SUBROUTINE usr_def_zgr()           ! Coriolis parameter  (if domain not on the sphere)
       ! subroutine for vertical grid
   END SUBROUTINE usr_def_zgr

 
   SUBROUTINE usr_def_ini( pts )
      !!----------------------------------------------------------------------
      !!                   ***  ROUTINE usr_def_ini  ***
      !! 
      !! ** Purpose :   Initialization of the dynamics and tracers
      !!                Here GYRE configuration example : (double gyre with rotated domain)
      !!
      !! ** Method  : - set temprature field
      !!              - set salinity   field
      !!----------------------------------------------------------------------
      REAL(wp), DIMENSION(jpi,jpj,jpk,jpts), INTENT(  out) ::   pts   ! T & S data
      !
      INTEGER :: ji, jj, jk  ! dummy loop indices
      !!----------------------------------------------------------------------
      !
      IF(lwp) WRITE(numout,*)
      IF(lwp) WRITE(numout,*) 'usr_def_ini : analytical definition of initial state '
      IF(lwp) WRITE(numout,*) '              T and S profiles deduced from LEVITUS '
      IF(lwp) WRITE(numout,*) '~~~~~~~~~~~'
      !
      DO jk = 1, jpk
         DO jj = 1, jpj
            DO ji = 1, jpi
               pts(ji,jj,jk,jp_tem) =  (  (  16. - 12. * TANH( (gdept_n(ji,jj,jk) - 400) / 700 ) )   &
                    &           * (-TANH( (500-gdept_n(ji,jj,jk)) / 150 ) + 1) / 2               &
                    &           + ( 15. * ( 1. - TANH( (gdept_n(ji,jj,jk)-50.) / 1500.) )        &
                    &           - 1.4 * TANH((gdept_n(ji,jj,jk)-100.) / 100.)                    &
                    &           + 7.  * (1500. - gdept_n(ji,jj,jk)) / 1500.)                     &
                    &           * (-TANH( (gdept_n(ji,jj,jk) - 500) / 150) + 1) / 2  ) * tmask(ji,jj,jk)

               pts(ji,jj,jk,jp_sal) =  (  (  36.25 - 1.13 * TANH( (gdept_n(ji,jj,jk) - 305) / 460 ) )  &
                    &         * (-TANH((500 - gdept_n(ji,jj,jk)) / 150) + 1) / 2              &
                    &         + ( 35.55 + 1.25 * (5000. - gdept_n(ji,jj,jk)) / 5000.          &
                    &         - 1.62 * TANH( (gdept_n(ji,jj,jk) - 60.  ) / 650. )             &
                    &         + 0.2  * TANH( (gdept_n(ji,jj,jk) - 35.  ) / 100. )             &
                    &         + 0.2  * TANH( (gdept_n(ji,jj,jk) - 1000.) / 5000.) )           &
                    &         * (-TANH((gdept_n(ji,jj,jk) - 500) / 150) + 1) / 2  ) * tmask(ji,jj,jk)
            END DO
         END DO
      END DO
      !   
   END SUBROUTINE usr_def_ini

   !!======================================================================
END MODULE usrdef
